/***********************************************************
Program mapmult.c
This program multiplies a floating point ascii Grass map by argument _factor_.
Command line arguments are _in_fname_, _factor_ and _out_fname_.
******************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <math.h>
#include <float.h>
#include <string.h>
#include <malloc.h>

char *ProgramName;
long ftwoalloc();
void ftwofree();

usage ()
{
   fprintf (stderr, "usage:  %s [orig_mapname] [factor] [new_mapname]\n\n", ProgramName);
   fprintf (stderr, "    where: [orig_mapname] is the name of the floating point ascii GRASS map\n");
   fprintf (stderr, "           [factor] is the constant which the orig. map must be multiplied by\n");
   fprintf (stderr, "to produce [float_mapname] floating point ascii GRASS map.\n");
   fprintf (stderr, "\n");
   exit (1);
}


int main(argc, argv)
int     argc;
char    *argv[];
{
   extern int errno;
   char *in_fname=NULL; 
   char *out_fname=NULL;
   char *c_factor=NULL;
   float factor;

   FILE *in_fptr;
   FILE *out_fptr;
   long i, j ;
   int did;
   long yes_intmap=0;
   int size;
   float **orig;
   float **new;
   int left,right;  /* these store the number of digits to l/r of decimal */
   float max=0;
   long rows,cols;
   double north,south,east,west;
   char northc[20],southc[20],eastc[20],westc[20],rowc[20],colc[20];
   long ltmp;
   float ftmp;
   
   ProgramName = argv[0];
   printf("argc=%d\n",argc);
   if(argc!=4) usage();
   
   in_fname = argv[1];
   c_factor = argv[2];
   out_fname = argv[3];
   factor=(float)atof(c_factor);
   size=strlen(c_factor);

   if((in_fptr=fopen(in_fname,"r"))==NULL)  
      {
      printf("\n\n\nCan't open file %s\n",in_fname);
      usage();
      }
   if((out_fptr=fopen(out_fname,"w"))==NULL)  
      {
      printf("\n\n\nCan't open file %s\n",out_fname);
      usage();
      }

   fscanf(in_fptr,"%s %lf",northc,&north);
   fscanf(in_fptr,"%s %lf",southc,&south);
   fscanf(in_fptr,"%s %lf",eastc,&east);
   fscanf(in_fptr,"%s %lf",westc,&west);
   fscanf(in_fptr,"%s %ld",rowc,&rows);
   fscanf(in_fptr,"%s %ld",colc,&cols);

   did=ftwoalloc(rows,cols,&orig);
   if(did){printf("problem allocating memory for input array\n");exit(-1);}

   did=ftwoalloc(rows,cols,&new);
   if(did){printf("problem allocating memory for output array\n");exit(-1);}

   printf("Converting orig. map %s to new map %s by multiplying by %f\n",
           in_fname,out_fname,factor);

   printf("%6s %lf\n",northc,north);
   printf("%6s %lf\n",southc,south);
   printf("%5s %lf\n",eastc,east);
   printf("%5s %lf\n",westc,west);
   printf("%5s %ld\n",rowc,rows);
   printf("%5s %ld\n",colc,cols);

   for(i=0;i<rows;i++)
      {
      for(j=0;j<cols;j++)
         {
         fscanf(in_fptr,"%f",&ftmp);
         orig[i][j]=ftmp;
         new[i][j]=orig[i][j]*factor;
         if(new[i][j]>max) max=new[i][j];
         if(new[i][j]==(float)((int)new[i][j])) yes_intmap++;
         }
      }
   if(yes_intmap==(rows*cols)) yes_intmap=1;
   else                        yes_intmap=0;
   
   if(max>0.0) right=(int)log10(max)+1;
   if(max<0.0) right=2;

   fprintf(out_fptr,"%6s %.2lf\n",northc,north);
   fprintf(out_fptr,"%6s %.2lf\n",southc,south);
   fprintf(out_fptr,"%5s  %.2lf\n",eastc,east);
   fprintf(out_fptr,"%5s  %.2lf\n",westc,west);
   fprintf(out_fptr,"%5s  %ld\n",rowc,rows);
   fprintf(out_fptr,"%5s  %ld\n",colc,cols);
   for(i=0;i<rows;i++)
      {
      for(j=0;j<cols;j++)
         {
         ftmp=new[i][j];
         if(yes_intmap==0)
            {
            if(right<=1) fprintf(out_fptr,"%8.5f ",ftmp);
            else if(right==2) fprintf(out_fptr,"%9.5f ",ftmp);
            else if(right==3) fprintf(out_fptr,"%10.5f ",ftmp);
            else if(right==4) fprintf(out_fptr,"%11.5f ",ftmp);
            else if(right==5) fprintf(out_fptr,"%11.4f ",ftmp);
            else if(right==6) fprintf(out_fptr,"%11.3f ",ftmp);
            else if(right==7) fprintf(out_fptr,"%11.2f ",ftmp);
            else if(right==8) fprintf(out_fptr,"%11.1f ",ftmp);
            else fprintf(out_fptr,"%f ",ftmp);
            }
         if(yes_intmap==1)
            {
            if(right<=1) fprintf(out_fptr,"%3.1f ",ftmp);
            else if(right==2) fprintf(out_fptr,"%4.1f ",ftmp);
            else if(right==3) fprintf(out_fptr,"%5.1f ",ftmp);
            else if(right==4) fprintf(out_fptr,"%6.1f ",ftmp);
            else if(right==5) fprintf(out_fptr,"%7.1f ",ftmp);
            else if(right==6) fprintf(out_fptr,"%8.1f ",ftmp);
            else if(right==7) fprintf(out_fptr,"%9.1f ",ftmp);
            else if(right==8) fprintf(out_fptr,"%10.1f ",ftmp);
            else fprintf(out_fptr,"%f ",ftmp);
            }
         }
      fprintf(out_fptr,"\n");
      }

   fclose(in_fptr);
   fclose(out_fptr);

   ftwofree(rows,&orig);
   ftwofree(rows,&new);
   printf("finished.\n");
   return(0);

}

long ftwoalloc(rows,cols,array)
long rows,cols;
float ***array;
{
   long i,j,k,frows,fcols;
   frows=rows+1;  /* added one for FORTRAN numbering */
   fcols=cols+1;  /* added one for FORTRAN numbering */

   *array=(float **)malloc(frows*sizeof(float *));
   for(i=0;i<frows;i++)
      {
      (*array)[i]=(float *)malloc(fcols*sizeof(float));
      if((*array)[i]==NULL) return(-1);
      }
   return(0);
}


void ftwofree(rows,array)
long rows;
float ***array;
{
   long i,frows;
   frows=rows+1;
   for(i=0;i<frows;i++)
      {
      free((*array)[i]);
      }
   free(*array);
}



